function Butterfly(descr){
    this.loc = [0,0,0,1.0];
    this.rot = [0,0,0,0];
    this.vel = [0,0,0,0];
    this.wingAngle = 0;
    this.flapDir = 1;
    this.flapSpeed = 10;
    this.size=1;
    for(var prop in descr){
	this[prop] = descr[prop];
    }
    this.init();
}



Butterfly.prototype.init = function (){
    var s = this.size;
    var upperleft = new Cube({"vColors": this.ulcols || this.color});
    upperleft.scale(s*1.0,s*1.0,s*0.2);
    var lowerleft = new Cube({"vColors": this.llcols || this.color});
    lowerleft.scale(s*1.5,s*1.5,s*0.2);
    
    var upperright = new Cube({"vColors": this.urcols || this.ulcols || this.color});
    upperright.scale(s*1.0,s*1.0,s*0.2);
    var lowerright = new Cube({"vColors": this.lrcols || this.llcols || this.color});
    lowerright.scale(s*1.5,s*1.5,s*0.2);

    upperleft.translate(-0.6 *s,-0.60*s,0);
    upperright.translate(0.6 *s,-0.60*s,0);
    lowerleft.translate(-0.85*s, 0.85*s,0);
    lowerright.translate(0.85*s, 0.85*s,0);
    
    

    this.wings = [upperleft,lowerleft,upperright,lowerright];
    //this.direct([0,0,1,0]);
    if(this.startVel){
	if(length(this.startVel) > 0)
	    this.direct(normalize(this.startVel));
	this.vel = this.startVel;
	}
    if(this.startLoc)
	this.translate(this.startLoc);

};

Butterfly.prototype.update = function(du){
    this.updateWings(du);
    this.translate(scale(du,this.vel));
    if(length(this.loc) > 10){
	this.updateVel(negate(this.vel));
    }
};

Butterfly.prototype.translate = function(vec){
    for(var part in this.wings){
	this.wings[part].translate(vec);
    }
    this.loc = add(this.loc,vec);
    this.loc[3] = 1;
};



Butterfly.prototype.updateWings = function(du){
    if (Math.abs(this.wingAngle) > 45){
	this.flapDir = -this.flapDir;
    }
    this.increaseWingAngle(this.flapDir*this.flapSpeed);


};

Butterfly.prototype.increaseWingAngle = function(angle){

    this.wingAngle += angle;
    var rot = this.rot;
    var loc = this.loc;
    //this.translate(negate(loc));
    //this.rotate(negate(rot));
    if(length(this.vel) > 0){
	var dir = normalize(this.vel) ;
    } else {
	if(length(this.rot) > 0){
	    var rotM = mat4();
	    var rotM = mult(rotate(this.rot[0],[1,0,0,0]),rotM);
	    var rotM = mult(rotate(this.rot[1],[0,1,0,0]),rotM);
	    var rotM = mult(rotate(this.rot[2],[0,0,1,0]),rotM);
	    var dir = normalize(mulMV(rotM,[0,1,0,0]));
	    }
	else{
	    var dir = [0,1,0,0];
	    }
	//debugger;

    };
	this.wings[0].rotateAround(this.loc,angle, dir);
	this.wings[1].rotateAround(this.loc,angle, dir);
	this.wings[2].rotateAround(this.loc,-angle,dir);
	this.wings[3].rotateAround(this.loc,-angle,dir);
    //this.rotate(rot);
    //this.translate(loc);
    
};

Butterfly.prototype.rotate = function(rotate,rotcenter){
    var center = rotcenter || this.loc;
    var angl = length(rotate);
   if(angl > 0){
        var rot2 = normalize(rotate);
	for(var part in this.wings){
		//this.wings[part].rotateAround(center,angl,rot2);
	        this.wings[part].rotateAround(center,rotate[0],[1,0,0,0]);
	        this.wings[part].rotateAround(center,rotate[1],[0,1,0,0]);
	        this.wings[part].rotateAround(center,rotate[2],[0,0,1,0]);
	    }
       }

    this.rot  = add(rotate,this.rot);
    //this.rot = [this.rot[0] % 360,this.rot[1] % 360,this.rot[2] % 360,0];
};

Butterfly.prototype.direct = function(newDirection){
    var cr = cross(newDirection,[0,1,0,0]);
    var angle = Math.asin(length(cr))*180/Math.PI;
    this.rotate(negate(this.rot));
    var nrot = scale(-angle,normalize(cr));
    nrot[3] = 0;
    this.rotate(nrot);
};

Butterfly.prototype.updateVel = function(newVel){
    if(length(newVel) != 0){
	this.direct(normalize(newVel));
    };
    this.vel = newVel;
};

Butterfly.prototype.render = function (gl){
    for(var part in this.wings){
	    this.wings[part].render(gl);
	}
};
